@@ -98,10 +98,12 @@ putDNSFlags DNSFlags{..} = put16 word
 
     word = execState st 0
 
+-- XXX: Use question class when implemented
+--
 putQuestion :: Question -> SPut
 putQuestion Question{..} = putDomain qname
                            <> put16 (fromTYPE qtype)
-                           <> put16 1
+                           <> put16 classIN
 
 putResourceRecord :: ResourceRecord -> SPut
 putResourceRecord ResourceRecord{..} = mconcat [
@@ -166,6 +168,12 @@ putRData rd = case rd of
         , put8 a
         , putByteString k
         ]
+    (RD_NSEC3PARAM h f i s) -> mconcat
+        [ put8 h
+        , put8 f
+        , put16 i
+        , putByteStringWithLength s
+        ]
     UnknownRData bytes -> putByteString bytes
 
 putOData :: OData -> SPut
