@@ -80,6 +80,8 @@ module Network.DNS.Types (
   -- * EDNS0
   , EDNS0
   , defaultEDNS0
+  , maxUdpSize
+  , minUdpSize
   -- ** Accessors
   , udpSize
   , extRCODE
@@ -114,13 +116,14 @@ import Network.DNS.Imports
 type Domain = ByteString
 
 -- | Type for a mailbox encoded on the wire as a DNS name, but the first label
--- is conceptually the user name, and sometimes has internal '.' characters
--- that are not label separators.
+-- is conceptually the user name, and sometimes has contains internal periods
+-- that are not label separators. Therefore, in mailboxes \@ is used as the
+-- separator between the first and second labels.
 type Mailbox = ByteString
 
 ----------------------------------------------------------------
 
-#if __GLASGOW_HASKELL__ >= 800
+#if __GLASGOW_HASKELL__ >= 802
 -- | Types for resource records.
 newtype TYPE = TYPE {
     -- | From type to number.
@@ -352,6 +355,7 @@ data DNSError =
     -- | Network failure.
   | NetworkFailure IOException
     -- | Error is unknown
+  | DecodeError String
   | UnknownDNSError
   deriving (Eq, Show, Typeable)
 
@@ -418,7 +422,7 @@ data OPCODE
 
 ----------------------------------------------------------------
 
-#if __GLASGOW_HASKELL__ >= 800
+#if __GLASGOW_HASKELL__ >= 802
 -- | Response code including EDNS0's 12bit ones.
 newtype RCODE = RCODE {
     -- | From rcode to number.
@@ -596,6 +600,8 @@ toRCODEforHeader w = toRCODE (w .&. 0x0f)
 
 ----------------------------------------------------------------
 
+-- XXX: The Question really should also include the CLASS
+--
 -- | Raw data format for DNS questions.
 data Question = Question {
     qname  :: Domain -- ^ A domain name
@@ -646,7 +652,7 @@ data RData = RD_A IPv4           -- ^ IPv4 address
            | RD_DNSKEY Word16 Word8 Word8 ByteString
                                  -- ^ DNSKEY (RFC4034)
            --RD_NSEC3
-           --RD_NSEC3PARAM
+           | RD_NSEC3PARAM Word8 Word8 Word16 ByteString
            | RD_TLSA Word8 Word8 Word8 ByteString
                                  -- ^ TLSA (RFC6698)
            --RD_CDS
@@ -674,6 +680,10 @@ instance Show RData where
   show (RD_DS t a dt dv) = show t ++ " " ++ show a ++ " " ++ show dt ++ " " ++ hexencode dv
   show RD_NULL = "NULL"
   show (RD_DNSKEY f p a k) = show f ++ " " ++ show p ++ " " ++ show a ++ " " ++ b64encode k
+  show (RD_NSEC3PARAM h f i s) = show h ++ " " ++ show f ++ " " ++ show i ++ " " ++ showSalt s
+    where
+      showSalt ""    = "-"
+      showSalt salt  = hexencode salt
 
 hexencode :: ByteString -> String
 hexencode = BS.unpack . L.toStrict . L.toLazyByteString . L.byteStringHex
@@ -737,7 +747,7 @@ data EDNS0 = EDNS0 {
   , options  :: [OData]
   } deriving (Eq, Show)
 
-#if __GLASGOW_HASKELL__ >= 800
+#if __GLASGOW_HASKELL__ >= 802
 -- | Default information for EDNS0.
 --
 -- >>> defaultEDNS0
@@ -751,6 +761,22 @@ data EDNS0 = EDNS0 {
 defaultEDNS0 :: EDNS0
 defaultEDNS0 = EDNS0 4096 NoErr False []
 
+-- | Maximum UDP size. If 'udpSize' of 'EDNS0' is larger than this,
+--   'fromEDNS0' uses this value instead.
+--
+-- >>> maxUdpSize
+-- 16384
+maxUdpSize :: Word16
+maxUdpSize = 16384
+
+-- | Minimum UDP size. If 'udpSize' of 'EDNS0' is smaller than this,
+--   'fromEDNS0' uses this value instead.
+--
+-- >>> minUdpSize
+-- 512
+minUdpSize :: Word16
+minUdpSize = 512
+
 -- | Generating a resource record for the additional section based on EDNS0.
 -- 'DNSFlags' is not generated.
 -- Just set the same 'RCODE' to 'DNSFlags'.
@@ -759,7 +785,7 @@ fromEDNS0 edns = ResourceRecord name' type' class' ttl' rdata'
   where
     name'  = "."
     type'  = OPT
-    class' = udpSize edns
+    class' = maxUdpSize `min` (minUdpSize `max` udpSize edns)
     ttl0'   = fromIntegral (fromRCODE (extRCODE edns) .&. 0x0ff0) `shiftL` 20
     ttl'
       | dnssecOk edns = ttl0' `setBit` 15
@@ -779,7 +805,7 @@ toEDNS0 _ _ = Nothing
 
 ----------------------------------------------------------------
 
-#if __GLASGOW_HASKELL__ >= 800
+#if __GLASGOW_HASKELL__ >= 802
 -- | EDNS0 Option Code (RFC 6891).
 newtype OptCode = OptCode {
     -- | From option code to number.
